/*
 * TCPVIEW
 *
 * Author:	Martin Hunt
 *		Networks and Distributed Computing
 *		Computing & Communications
 *		University of Washington
 *		Administration Building, AG-44
 *		Seattle, WA  98195
 *		Internet: martinh@cac.washington.edu
 *
 *
 * Copyright 1992 by the University of Washington
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appears in all copies and that both the
 * above copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the University of Washington not be
 * used in advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  This software is made
 * available "as is", and
 * THE UNIVERSITY OF WASHINGTON DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED,
 * WITH REGARD TO THIS SOFTWARE, INCLUDING WITHOUT LIMITATION ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND IN
 * NO EVENT SHALL THE UNIVERSITY OF WASHINGTON BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, TORT
 * (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#ifndef lint
static char rcsid[] =
    "@(#) $Header: /usr/staff/martinh/tcpview/RCS/detail-arp.c,v 1.1 1993/04/22 20:14:57 martinh Exp $ (UW)";
#endif

#include <sys/param.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <net/if.h>
#include <netinet/in.h>
#include <netinet/if_ether.h>

#include "tcpview.h"
#include "addrtoname.h"

void
detail_arp(ap)
     register struct ether_arp *ap;
{
  int length;
  char *buf;
  u_short hard, pro, op;

  length = Phdr->caplen - Offset;
  if (length < sizeof(struct ether_arp)) {
    printf("***** ARP/RARP frame (truncated) *****");
    hex( -1, -1 );
    return;
  }
  printf("----- ARP/RARP frame -----\n\n");

  hard = ntohs(ap->arp_hrd);
  pro = ntohs(ap->arp_pro);
  op = ntohs(ap->arp_op);

  switch (hard) {
  case 1:
    buf = "10Mb Ethernet";
    break;
  case 6:
    buf= "FDDI";
    break;
  default:
    buf = "unknown";
  }
	
  printf("Hardware type = %d ( %s )\n",hard,buf);
  printf("Protocol type = %4s ( %s )\n",
	etherproto_str(ap->arp_pro,ADDR_NUMERICAL),
	etherproto_str(ap->arp_pro,0));
  printf("Hardware address length = %d bytes\n",ap->arp_hln);
  printf("Protocol address length = %d bytes\n",ap->arp_pln);

  switch (op) {
  case ARPOP_REQUEST:
    buf = "ARP request";
    break;
  case ARPOP_REPLY:
    buf = "ARP reply";
    break;
  case REVARP_REQUEST:
    buf = "RARP request";
    break;
  case REVARP_REPLY:
    buf = "RARP reply";
    break;
  default:
    buf = "unknown";
  }
  printf("Opcode %d ( %s )\n",op,buf);
  printf("Sender's hardware address = %s\n",etheraddr_str(SHA(ap),ADDR_NUMERICAL));
  printf("Sender's protocol address = %s,  %s\n",
	ipaddr_str(SPA(ap),ADDR_NUMERICAL),
	ipaddr_str(SPA(ap),0));
  printf("Target hardware address = %s\n",etheraddr_str(THA(ap),ADDR_NUMERICAL));
  printf("Target protocol address = %s,  %s\n",
	ipaddr_str(TPA(ap),ADDR_NUMERICAL),
	ipaddr_str(TPA(ap),0));
  printf("\n");

  hex( 0, sizeof(struct ether_arp)-1);
  hex( -1, -1 );
  hex( (u_long)(&ap->arp_hrd) - (u_long)ap, 
      (u_long)(&ap->arp_hrd) - (u_long)ap + sizeof(ap->arp_hrd)-1 );
  hex( (u_long)(&ap->arp_pro) - (u_long)ap,
      (u_long)(&ap->arp_pro) - (u_long)ap + sizeof(ap->arp_pro)-1 );
  hex( (u_long)(&ap->arp_hln) - (u_long)ap,
      (u_long)(&ap->arp_hln) - (u_long)ap + sizeof(ap->arp_hln)-1 );
  hex( (u_long)(&ap->arp_pln) - (u_long)ap,
      (u_long)(&ap->arp_pln) - (u_long)ap + sizeof(ap->arp_pln)-1 );
  hex( (u_long)(&ap->arp_op) - (u_long)ap,
      (u_long)(&ap->arp_op) - (u_long)ap + sizeof(ap->arp_op)-1 );
  hex( (u_long)(&ap->arp_sha) - (u_long)ap,
      (u_long)(&ap->arp_sha) - (u_long)ap + sizeof(ap->arp_sha)-1 );
  hex( (u_long)(ap->arp_spa) - (u_long)ap,
      (u_long)(ap->arp_spa) - (u_long)ap + sizeof(ap->arp_spa)-1 );
  hex( (u_long)(&ap->arp_tha) - (u_long)ap,
      (u_long)(&ap->arp_tha) - (u_long)ap + sizeof(ap->arp_tha)-1 );
  hex( (u_long)(ap->arp_tpa) - (u_long)ap,
      (u_long)(ap->arp_tpa) - (u_long)ap + sizeof(ap->arp_tpa)-1 );
  hex( -1, -1 );
}




